@@ -410,11 +410,7 @@ QDomElement XdgMenu::findMenu(QDomElement& baseElement, const QString& path, boo
         return QDomElement();
 
 
-#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
     const QStringList names = path.split(QLatin1Char('/'), Qt::SkipEmptyParts);
-#else
-    const QStringList names = path.split(QLatin1Char('/'), QString::SkipEmptyParts);
-#endif
     QDomElement el = baseElement;
     for (const QString &n : names)
     {
@@ -541,12 +537,12 @@ void XdgMenuPrivate::processDirectoryEntries(QDomElement& element, const QString
     dirs << parentDirs;
 
     bool found = false;
-    for (const QString &file : qAsConst(files)){
+    for (const QString &file : std::as_const(files)){
         if (file.startsWith(QLatin1Char('/')))
             found = loadDirectoryFile(file, element);
         else
         {
-            for (const QString &dir : qAsConst(dirs))
+            for (const QString &dir : std::as_const(dirs))
             {
                 found = loadDirectoryFile(dir + QLatin1Char('/') + file, element);
                 if (found) break;
@@ -675,7 +671,7 @@ QString XdgMenu::getMenuFileName(const QString& baseName)
 
     for (const QString &configDir : configDirs)
     {
-        for (const QString &f : qAsConst(wellKnownFiles))
+        for (const QString &f : std::as_const(wellKnownFiles))
         {
             QFileInfo file(QString::fromLatin1("%1/menus/%2").arg(configDir, f));
             if (file.exists())
