@@ -57,9 +57,6 @@
 
 #include <private/qhexstring_p.h>
 
-//QT_BEGIN_NAMESPACE
-
-
 Q_GLOBAL_STATIC(XdgIconLoader, iconLoaderInstance)
 
 /* Theme to use in last resort, if the theme does not have the icon, neither the parents  */
@@ -109,7 +106,7 @@ class QIconCacheGtkReader
 {
 public:
     explicit QIconCacheGtkReader(const QString &themeDir);
-    QVector<const char *> lookup(QStringView);
+    QList<const char *> lookup(QStringView);
     bool isValid() const { return m_isValid; }
     bool reValid(bool infoRefresh);
 private:
@@ -149,14 +146,12 @@ QIconCacheGtkReader::QIconCacheGtkReader(const QString &dirName)
     // Note: The cache file can be (IS) removed and newly created during the
     // cache update. But we hold open file descriptor for the "old" removed
     // file. So we need to watch the changes and reopen/remap the file.
-    QObject::connect(gtkCachesWatcher(), &QFileSystemWatcher::fileChanged, &m_file, [this](const QString & path)
+    gtkCachesWatcher->addPath(dirName);
+    QObject::connect(gtkCachesWatcher(), &QFileSystemWatcher::directoryChanged, &m_file, [this]
         {
-            if (m_file.fileName() == path)
-            {
-                m_isValid = false;
-                // invalidate icons to reload them ...
-                QIconLoader::instance()->invalidateKey();
-            }
+            m_isValid = false;
+            // invalidate icons to reload them ...
+            QIconLoader::instance()->invalidateKey();
         });
     reValid(false);
 }
@@ -175,11 +170,6 @@ bool QIconCacheGtkReader::reValid(bool infoRefresh)
     if (!m_cacheFileInfo.exists() || m_cacheFileInfo.lastModified() < QFileInfo{dir.absolutePath()}.lastModified())
         return m_isValid;
 
-    // Note: If the file is removed, it is also silently removed from watched
-    // paths in QFileSystemWatcher.
-    if (!gtkCachesWatcher->files().contains(m_cacheFileInfo.absoluteFilePath()))
-        gtkCachesWatcher->addPath(m_cacheFileInfo.absoluteFilePath());
-
     if (!m_file.open(QFile::ReadOnly))
         return m_isValid;
     m_size = m_file.size();
@@ -220,9 +210,9 @@ static quint32 icon_name_hash(const char *p)
     For example, this would return { "32x32/apps", "24x24/apps" , ... }
  */
 
-QVector<const char *> QIconCacheGtkReader::lookup(QStringView name)
+QList<const char *> QIconCacheGtkReader::lookup(QStringView name)
 {
-    QVector<const char *> ret;
+    QList<const char *> ret;
     if (!isValid() || name.isEmpty())
         return ret;
 
@@ -404,14 +394,14 @@ QThemeIconInfo XdgIconLoader::findIconHelper(const QString &themeName,
     QStringView iconNameFallback(iconName);
 
     // Iterate through all icon's fallbacks in current theme
-    if (info.entries.isEmpty()) {
+    if (info.entries.empty()) {
         const QString svgIconName = iconNameFallback + svgext;
         const QString pngIconName = iconNameFallback + pngext;
         const QString xpmIconName = iconNameFallback + xpmext;
 
         // Add all relevant files
         for (int i = 0; i < contentDirs.size(); ++i) {
-            QVector<QIconDirInfo> subDirs = theme.keyList();
+            QList<QIconDirInfo> subDirs = theme.keyList();
 
             // Try to reduce the amount of subDirs by looking in the GTK+ cache in order to save
             // a massive amount of file stat (especially if the icon is not there)
@@ -419,7 +409,7 @@ QThemeIconInfo XdgIconLoader::findIconHelper(const QString &themeName,
             if (cache->isValid() || cache->reValid(true)) {
                 const auto result = cache->lookup(iconNameFallback);
                 if (cache->isValid()) {
-                    const QVector<QIconDirInfo> subDirsCopy = subDirs;
+                    const QList<QIconDirInfo> subDirsCopy = subDirs;
                     subDirs.clear();
                     subDirs.reserve(result.count());
                     for (const char *s : result) {
@@ -440,38 +430,38 @@ QThemeIconInfo XdgIconLoader::findIconHelper(const QString &themeName,
                 const QString subDir = contentDir + dirInfo.path + QLatin1Char('/');
                 const QString pngPath = subDir + pngIconName;
                 if (QFile::exists(pngPath)) {
-                    PixmapEntry *iconEntry = new PixmapEntry;
+                    auto iconEntry = std::make_unique<PixmapEntry>();
                     iconEntry->dir = dirInfo;
                     iconEntry->filename = pngPath;
                     // Notice we ensure that pixmap entries always come before
                     // scalable to preserve search order afterwards
-                    info.entries.prepend(iconEntry);
-                } else {
+                    info.entries.insert(info.entries.begin(), std::move(iconEntry));
+                } else if (gSupportsSvg) {
                     const QString svgPath = subDir + svgIconName;
-                    if (gSupportsSvg && QFile::exists(svgPath)) {
-                        ScalableEntry *iconEntry = (followColorScheme() && theme.followsColorScheme()) ? new ScalableFollowsColorEntry : new ScalableEntry;
+                    if (QFile::exists(svgPath)) {
+                        std::unique_ptr<ScalableEntry> iconEntry = (followColorScheme() && theme.followsColorScheme()) ? std::make_unique<ScalableFollowsColorEntry>() : std::make_unique<ScalableEntry>();
                         iconEntry->dir = dirInfo;
                         iconEntry->filename = svgPath;
-                        info.entries.append(iconEntry);
+                        info.entries.push_back(std::move(iconEntry));
                     }
                 }
                 const QString xpmPath = subDir + xpmIconName;
                 if (QFile::exists(xpmPath)) {
-                    PixmapEntry *iconEntry = new PixmapEntry;
+                    auto iconEntry = std::make_unique<PixmapEntry>();
                     iconEntry->dir = dirInfo;
                     iconEntry->filename = xpmPath;
                     // Notice we ensure that pixmap entries always come before
                     // scalable to preserve search order afterwards
-                    info.entries.append(iconEntry);
+                    info.entries.insert(info.entries.begin(), std::move(iconEntry));
                 }
             }
         }
 
-        if (!info.entries.isEmpty())
+        if (!info.entries.empty())
             info.iconName = iconNameFallback.toString();
     }
 
-    if (info.entries.isEmpty()) {
+    if (info.entries.empty()) {
         const QStringList parents = theme.parents();
         // Search recursively through inherited themes
         for (int i = 0 ; i < parents.size() ; ++i) {
@@ -481,44 +471,44 @@ QThemeIconInfo XdgIconLoader::findIconHelper(const QString &themeName,
             if (!visited.contains(parentTheme)) // guard against recursion
                 info = findIconHelper(parentTheme, iconName, visited);
 
-            if (!info.entries.isEmpty()) // success
+            if (!info.entries.empty()) // success
                 break;
         }
 
         // make sure that hicolor is also searched before dash fallbacks
-        if (info.entries.isEmpty()
+        if (info.entries.empty()
             && !parents.contains(QLatin1String("hicolor"))
             && !visited.contains(QLatin1String("hicolor"))) {
             info = findIconHelper(QLatin1String("hicolor"), iconName, visited);
         }
     }
 
-    if (info.entries.isEmpty()) {
+    if (info.entries.empty()) {
         // Also, consider Qt's fallback search paths (which are not defined by Freedesktop)
         // if the icon is not found in any inherited theme
         const auto fallbackPaths = QIcon::fallbackSearchPaths();
         for (const auto &fallbackPath : fallbackPaths) {
             const QString pngPath = fallbackPath + QLatin1Char('/') + iconName + pngext;
             if (QFile::exists(pngPath)) {
-                PixmapEntry *iconEntry = new PixmapEntry;
+                auto iconEntry = std::make_unique<PixmapEntry>();
                 QIconDirInfo dirInfo(fallbackPath);
                 iconEntry->dir = dirInfo;
                 iconEntry->filename = pngPath;
-                info.entries.prepend(iconEntry);
+                info.entries.insert(info.entries.begin(), std::move(iconEntry));
             } else {
                 const QString svgPath = fallbackPath + QLatin1Char('/') + iconName + svgext;
                 if (gSupportsSvg && QFile::exists(svgPath)) {
-                    ScalableEntry *iconEntry = new ScalableEntry;
+                    auto iconEntry = std::make_unique<ScalableEntry>();
                     QIconDirInfo dirInfo(fallbackPath);
                     iconEntry->dir = dirInfo;
                     iconEntry->filename = svgPath;
-                    info.entries.append(iconEntry);
+                    info.entries.push_back(std::move(iconEntry));
                 }
             }
         }
     }
 
-    if (dashFallback && info.entries.isEmpty()) {
+    if (dashFallback && info.entries.empty()) {
         // If it's possible - find next fallback for the icon
         const int indexOfDash = iconNameFallback.lastIndexOf(QLatin1Char('-'));
         if (indexOfDash != -1) {
@@ -543,22 +533,22 @@ QThemeIconInfo XdgIconLoader::unthemedFallback(const QString &iconName, const QS
         QDir currentDir(contentDir);
 
         if (currentDir.exists(iconName + pngext)) {
-            PixmapEntry *iconEntry = new PixmapEntry;
+            auto iconEntry = std::make_unique<PixmapEntry>();
             iconEntry->filename = currentDir.filePath(iconName + pngext);
             // Notice we ensure that pixmap entries always come before
             // scalable to preserve search order afterwards
-            info.entries.prepend(iconEntry);
+            info.entries.insert(info.entries.begin(), std::move(iconEntry));
         } else if (gSupportsSvg &&
             currentDir.exists(iconName + svgext)) {
-            ScalableEntry *iconEntry = new ScalableEntry;
+            auto iconEntry = std::make_unique<ScalableEntry>();
             iconEntry->filename = currentDir.filePath(iconName + svgext);
-            info.entries.append(iconEntry);
+            info.entries.push_back(std::move(iconEntry));
         } else if (currentDir.exists(iconName + xpmext)) {
-            PixmapEntry *iconEntry = new PixmapEntry;
+            auto iconEntry = std::make_unique<PixmapEntry>();
             iconEntry->filename = currentDir.filePath(iconName + xpmext);
             // Notice we ensure that pixmap entries always come before
             // scalable to preserve search order afterwards
-            info.entries.append(iconEntry);
+            info.entries.insert(info.entries.begin(), std::move(iconEntry));
         }
     }
     return info;
@@ -570,13 +560,13 @@ QThemeIconInfo XdgIconLoader::loadIcon(const QString &name) const
     if (!theme_name.isEmpty()) {
         QStringList visited;
         auto info = findIconHelper(theme_name, name, visited, true);
-        if (info.entries.isEmpty()) {
-            const auto unthemedInfo = unthemedFallback(name, QIcon::themeSearchPaths());
-            if (unthemedInfo.entries.isEmpty()) {
+        if (info.entries.empty()) {
+            auto unthemedInfo = unthemedFallback(name, QIcon::themeSearchPaths());
+            if (unthemedInfo.entries.empty()) {
                 /* Freedesktop standard says to look in /usr/share/pixmaps last */
                 const QStringList pixmapPath = (QStringList() << QString::fromLatin1("/usr/share/pixmaps"));
-                const auto pixmapInfo = unthemedFallback(name, pixmapPath);
-                if (pixmapInfo.entries.isEmpty()) {
+                auto pixmapInfo = unthemedFallback(name, pixmapPath);
+                if (pixmapInfo.entries.empty()) {
                     return QThemeIconInfo();
                 } else {
                     return pixmapInfo;
@@ -601,10 +591,7 @@ XdgIconLoaderEngine::XdgIconLoaderEngine(const QString& iconName)
 {
 }
 
-XdgIconLoaderEngine::~XdgIconLoaderEngine()
-{
-    qDeleteAll(m_info.entries);
-}
+XdgIconLoaderEngine::~XdgIconLoaderEngine() = default;
 
 XdgIconLoaderEngine::XdgIconLoaderEngine(const XdgIconLoaderEngine &other)
         : QIconEngine(other),
@@ -631,18 +618,13 @@ bool XdgIconLoaderEngine::write(QDataStream &out) const
 
 bool XdgIconLoaderEngine::hasIcon() const
 {
-    return !(m_info.entries.isEmpty());
+    return !(m_info.entries.empty());
 }
 
 // Lazily load the icon
 void XdgIconLoaderEngine::ensureLoaded()
 {
-    if (!(QIconLoader::instance()->themeKey() == m_key)) {
-        qDeleteAll(m_info.entries);
-        m_info.entries.clear();
-        m_info.iconName.clear();
-
-        Q_ASSERT(m_info.entries.empty());
+    if (QIconLoader::instance()->themeKey() != m_key) {
         m_info = XdgIconLoader::instance()->loadIcon(m_iconName);
         m_key = QIconLoader::instance()->themeKey();
     }
@@ -711,32 +693,28 @@ static int directorySizeDistance(const QIconDirInfo &dir, int iconsize, int icon
     return INT_MAX;
 }
 
-QIconLoaderEngineEntry *XdgIconLoaderEngine::entryForSize(const QSize &size, int scale)
+QIconLoaderEngineEntry *XdgIconLoaderEngine::entryForSize(const QThemeIconInfo &info, const QSize &size, int scale)
 {
     int iconsize = qMin(size.width(), size.height());
 
-    // Note that m_info.entries are sorted so that png-files
+    // Note that info.entries are sorted so that png-files
     // come first
 
-    const int numEntries = m_info.entries.size();
-
     // Search for exact matches first
-    for (int i = 0; i < numEntries; ++i) {
-        QIconLoaderEngineEntry *entry = m_info.entries.at(i);
+    for (const auto &entry : info.entries) {
         if (directoryMatchesSize(entry->dir, iconsize, scale)) {
-            return entry;
+            return entry.get();
         }
     }
 
     // Find the minimum distance icon
     int minimalSize = INT_MAX;
     QIconLoaderEngineEntry *closestMatch = nullptr;
-    for (int i = 0; i < numEntries; ++i) {
-        QIconLoaderEngineEntry *entry = m_info.entries.at(i);
+    for (const auto &entry : info.entries) {
         int distance = directorySizeDistance(entry->dir, iconsize, scale);
         if (distance < minimalSize) {
             minimalSize  = distance;
-            closestMatch = entry;
+            closestMatch = entry.get();
         }
     }
     return closestMatch;
@@ -756,7 +734,7 @@ QSize XdgIconLoaderEngine::actualSize(const QSize &size, QIcon::Mode mode,
 
     ensureLoaded();
 
-    QIconLoaderEngineEntry *entry = entryForSize(size);
+    QIconLoaderEngineEntry *entry = entryForSize(m_info, size);
     if (entry) {
         const QIconDirInfo &dir = entry->dir;
         if (dir.type == QIconDirInfo::Scalable || dynamic_cast<ScalableEntry *>(entry))
@@ -961,7 +939,7 @@ QPixmap XdgIconLoaderEngine::pixmap(const QSize &size, QIcon::Mode mode,
 {
     ensureLoaded();
 
-    QIconLoaderEngineEntry *entry = entryForSize(size);
+    QIconLoaderEngineEntry *entry = entryForSize(m_info, size);
     if (entry)
         return entry->pixmap(size, mode, state);
 
@@ -973,53 +951,45 @@ QString XdgIconLoaderEngine::key() const
     return QLatin1String("XdgIconLoaderEngine");
 }
 
-void XdgIconLoaderEngine::virtual_hook(int id, void *data)
+QString XdgIconLoaderEngine::iconName()
 {
     ensureLoaded();
-
-    switch (id) {
-    case QIconEngine::AvailableSizesHook:
-        {
-            QIconEngine::AvailableSizesArgument &arg
-                    = *reinterpret_cast<QIconEngine::AvailableSizesArgument*>(data);
-            const int N = m_info.entries.size();
-            QList<QSize> sizes;
-            sizes.reserve(N);
-
-            // Gets all sizes from the DirectoryInfo entries
-            for (int i = 0; i < N; ++i) {
-                int size = m_info.entries.at(i)->dir.size;
-                sizes.append(QSize(size, size));
-            }
-            arg.sizes.swap(sizes); // commit
-        }
-        break;
-    case QIconEngine::IconNameHook:
-        {
-            QString &name = *reinterpret_cast<QString*>(data);
-            name = m_info.iconName;
-        }
-        break;
-    case QIconEngine::IsNullHook:
-        {
-            *reinterpret_cast<bool*>(data) = m_info.entries.isEmpty();
-        }
-        break;
-    case QIconEngine::ScaledPixmapHook:
-        {
-            QIconEngine::ScaledPixmapArgument &arg = *reinterpret_cast<QIconEngine::ScaledPixmapArgument*>(data);
-            // QIcon::pixmap() multiplies size by the device pixel ratio.
-            const int integerScale = qCeil(arg.scale);
-            QIconLoaderEngineEntry *entry = entryForSize(arg.size / integerScale, integerScale);
-            arg.pixmap = entry ? entry->pixmap(arg.size, arg.mode, arg.state) : QPixmap();
-        }
-        break;
-    default:
-        QIconEngine::virtual_hook(id, data);
-    }
+    return m_info.iconName;
 }
 
+bool XdgIconLoaderEngine::isNull()
+{
+    ensureLoaded();
+    return m_info.entries.empty();
+}
 
-//QT_END_NAMESPACE
+QPixmap XdgIconLoaderEngine::scaledPixmap(const QSize &size, QIcon::Mode mode, QIcon::State state, qreal scale)
+{
+    ensureLoaded();
+    const int integerScale = qCeil(scale);
+    QIconLoaderEngineEntry *entry = entryForSize(m_info, size / integerScale, integerScale);
+    return entry ? entry->pixmap(size, mode, state) : QPixmap();
+}
+
+QList<QSize> XdgIconLoaderEngine::availableSizes(QIcon::Mode mode, QIcon::State state)
+{
+    Q_UNUSED(mode);
+    Q_UNUSED(state);
+    ensureLoaded();
+    const int N = m_info.entries.size();
+    QList<QSize> sizes;
+    sizes.reserve(N);
+
+    // Gets all sizes from the DirectoryInfo entries
+    for (const auto &entry : m_info.entries) {
+        if (entry->dir.type == QIconDirInfo::Fallback) {
+            sizes.append(QIcon(entry->filename).availableSizes());
+        } else {
+            int size = entry->dir.size;
+            sizes.append(QSize(size, size));
+        }
+    }
+    return sizes;
+}
 
 #endif //QT_NO_ICON
